var Koa = require("koa")
var debug = require("util").debuglog("nyu")
var utils = require("../utils")
var path = require("path")
var fs = require("fs")
var http = require("http")

module.exports = function ($nyu, $package, $config, $logManager, $koaSession, $koaSecurity) {
	var app = new Koa();
	app.name = $package.name
	app.poweredBy = false
	app.keys = $config.get("security::keys")

	debug("patching in request helper object")
	Object.defineProperty(app.context, "logger", {
		enumerable: true,
		value: $logManager.getLogger("application")
	})
	Object.defineProperty(app.context, "helper", {
		enumerable: true,
		value: utils.requireDir($nyu.resolve("app/request-helpers"), true)
	})
	// TODO patch qs and/or csrf? handled by router?
	require("../koa-patches")(app)

	// jade view engine
	var options = $config.get("views")
	options.basedir = options.viewPath = "app/views"
	// options.locals
	options.noCache = options.debug = app.env === "development"
	if (!options.helperPath) {
		options.helperPath = []
	}
	if (fs.existsSync($nyu.resolve("app/view-helpers"))) {
		options.helperPath.push("app/view-helpers")
	}
	options.helperPath.push({ nyu: path.join(__dirname, "../view-helper.js") })

	app.use(require("koa-jade").middleware(options))
	app.use(require("koa-error-callback")(function* (error) {
		error.statusText = http.STATUS_CODES[error.status]
		yield this.render($config.get("views::errorView"), {
			error: error,
			ctx: this,
			debug: this.app.env === "development"
		})
	}))

	app.use(require("../middleware/logger")({ logger: $logManager.getLogger("http") }))

	options = $config.get("http::static")
	if (!options.disable) {
		options.hidden = false
		options.defer = false

		app.use(require("koa-static")($nyu.resolve("public"), options))
	}
	app.use(require("koa-slasher")({ enable308: true }))

	app.use(require("koa-bodyparser")())
	app.use($koaSession)
	app.use(require("../middleware/flash")())

	if ($koaSecurity) {
		app.use($koaSecurity)
	}

	app.use(require("koa-validate")())
	app.use(require("../router").create({
		path: $nyu.resolve("app/controllers"),
		routes: $config.get("routes")
	}))

	return app;
}
