var util = require("util")
var strformat = require('strformat')

/*
 * middleware
 */
module.exports = exports = function (opts) {
	if (!opts) {
		opts = { logger: console }
	}
	opts.format = opts.format || exports.common

	var log = (opts.logger.http? opts.logger.http: opts.logger.info).bind(opts.logger)
	var fmt = opts.format

	return function* (next) {
		var start = new Date()
		yield next
		var diff = Date.now() - start.getTime()

		log(strformat(fmt, {
			host:            this.request.host,
			"remote-addr":   this.request.ip,
			"remote-user":   this.user? this.user.toString(): "-",
			date:            start.toString(),
			method:          this.request.method,
			url:             this.request.url,
			"http-version":  this.req.httpVersion,
			status:          this.response.status,
			length:          this.response.length || "-",
			"response-time": diff,
			referrer:        this.request.header["referrer"] || this.request.header["referer"] || "-",
			"user-agent":    this.request.header["user-agent"] || "-"
		}))

		if (this.response.status >= 300 && this.response.status < 400) {
			log("Redirect from " + this.url + " to " + this.response.header["location"])
		}
	}
}

exports.common = "{host} {remote-addr} - {remote-user} [{date}] \"{method} {url} HTTP/{http-version}\" {status} {length}b {response-time}ms"
exports.combined = "{host} {remote-addr} - {remote-user} [{date}] \"{method} {url} HTTP/{http-version}\" {status} {length}b {response-time}ms \"{referrer}\" \"{user-agent}\""
exports.small = "{host} {remote-addr} - - [{date}] \"{method} {url}\" {status}b {length}"
exports.dev = "{method} {host}{url} {status}"
