var EventEmitter = require("events")
var path = require("path")
var util = require("util")
var debug = util.debuglog("nyu")
var minioc = require("minioc")
// TODO better ioc / bootable / ...?


function Nyu() {
	EventEmitter.call(this)

	minioc.register("nyu").as.singleton.value(this)
	minioc.register("package").as.value(this.require("package.json"))
	debug("creating nyu for %s", minioc.get("package").name)
	this.init()
}
util.inherits(Nyu, EventEmitter)

Nyu.prototype.init = function () {
	minioc.register("logManager").as.value(require("./components/logManager"))
	minioc.register("config").from.factory(require("./components/config"))
	minioc.register("koa").from.factory(require("./components/koa"))
	minioc.register("koaSession").from.factory(require("./components/koa-session"))
	minioc.register("koaSecurity").from.factory(require("./components/koa-security"))
	minioc.register("redis").from.factory(require("./components/redis"))
	minioc.register("database").from.factory(require("./components/thinky"))
}

Nyu.prototype.boot = function (done) {
	minioc.when("koa", function (koa) {
		koa.listen(3000, function () {
			if (done) {
				setImmediate(done)
			}
		})
	})
}

Nyu.prototype.resolve = function (file) {
	var args = Array.prototype.slice.call(arguments)
	args.unshift(process.cwd())

	return path.join.apply(path, args)
}

Nyu.prototype.require = function (file) {
	return require(this.resolve.apply(this, arguments))
}

Nyu.prototype.invoke = function (fn, options) {
	if (util.isString(fn)) {
		fn = this.require(fn)
	}

	debug("queueing function %s...", fn.name || "anonymous")
	setImmediate(function () {
		minioc.fulfill("nyu-invoke", fn, options)
	})
}

module.exports = exports = Nyu;
exports.boot = function (root, done) {
	process.chdir(root)
	var nyu = new Nyu()
	nyu.boot(done)
}


exports.Components = require("./utils").requireDir(path.join(__dirname, "components"), true)
