var Route = require("./route")
var path = require("path")
var debug = require("util").debuglog("nyu")

function NyuRouter(opts) {
	this.routes = {}

	if (!opts) {
		opts = { }
	}

	if (!opts.routes) {
		opts = { routes: opts }
	}

	this.basePath = opts.path || "controller"

	if (opts.routes) {
		this.registerRoutes(opts.routes)
	}
}


NyuRouter.prototype.registerRoute = function (name, route) {
	if (this.routes[name]) {
		throw new Error("route " + name + " already registered!")
	}

	route = Route.create(this, name, route)
	this.routes[name] = route
}

NyuRouter.prototype.registerRoutes = function (routes) {
	for (var name in routes) {
		this.registerRoute(name, routes[name])
	}
}

NyuRouter.prototype.generateRoute = function (name, params) {
	var route = this.routes[name]

	if (!route) {
		throw new Error("route " + name + " not found")
	}

	return route.generate(params)
}

NyuRouter.prototype.isRoute = function (name, url) {
	var route = this.routes[name]

	if (!route) {
		throw new Error("route " + name + " not found")
	}

	return route.parse(url)
}

NyuRouter.prototype.findRoute = function (method, url) {
	if (!url) {
		url = method
		method = "get"
	}
	method = method.toLowerCase()

	for (var name in this.routes) {
		debug("trying route %s", name)
		var result = this.routes[name].parse(method, url)

		if (result) {
			debug("route %s matching!", name)
			return result
		}
	}

	debug("no route matching!")
	return false
}

NyuRouter.prototype.redirect = function (name) {
	return this.registerRoute(name, { type: "redirect", method: "any" })
}

NyuRouter.prototype.resource = function (name) {
	return this.registerRoute(name, { type: "resource" })
}

NyuRouter.prototype.match = function (name) {
	return this.registerRoute(name, { type: "route", method: "any" })
}

NyuRouter.prototype.any = function (name) {
	return this.registerRoute(name, { type: "route", method: "any" })
}

NyuRouter.prototype.get = function (name) {
	return this.registerRoute(name, { type: "route", method: "get" })
}

NyuRouter.prototype.put = function (name) {
	return this.registerRoute(name, { type: "route", method: "put" })
}

NyuRouter.prototype.post = function (name) {
	return this.registerRoute(name, { type: "route", method: "post" })
}

NyuRouter.prototype.patch = function (name) {
	return this.registerRoute(name, { type: "route", method: "patch" })
}

NyuRouter.prototype.del = NyuRouter.prototype.delete = function (name) {
	return this.registerRoute(name, { type: "route", method: "delete" })
}

NyuRouter.prototype.middleware = function () {
	var r = this
	return function* router(next) {
		Object.defineProperty(this, "router", { value: r })
		debug("matching %s %s", this.method, this.url)

		// match
		var m = r.findRoute(this.method, this.url)
		if (!m) {
			return yield next
		}

		// fetch controller
		try {
			var ctrl = require(path.join(r.basePath, m.controller))

			if (ctrl._init) {
				next = yield* ctrl._init.call(this, next)
			}

			return yield ctrl[m.action]
		} catch (ex) {
			debug("controller failed to execute", ex)
			throw ex
		}
	}
}

module.exports = exports = NyuRouter

exports.create = function (opts) {
	var router = new NyuRouter(opts)
	return router.middleware()
}
