var methods = require("methods")
var util = require("util")
var strformat = require("strformat")
var xre = require("xregexp").XRegExp

var UrlRegExp = /\{([^\}]+)\}/ig

function Route(router, name, options) {
	this._router = router
	this.name = name
	this.meta = {}
	this._method = "any"

	if (options) {
		this.setOptions(options)
	}
}

Route.prototype.setOptions = function (options) {
	if (options.type) {
		delete options.type
	}

	for (var method in options) {
		if (this[method]) {
			this[method](options[method])
		} else {
			this.meta[method] = options[method]
		}
	}
}

Route.prototype.method = function (method) {
	if (method !== "any" && method !== "all" && methods.indexOf(method) < 0) {
		throw new Error("method " + method + " not supported")
	} else {
		if (method === "all") {
			method = "any"
		}
		this._method = method
	}

	return this
}

Route.prototype.from = Route.prototype.url = function (url) {
	this._url = url
	this.compileUrl()

	return this
}

Route.prototype.to = function (controller, action) {
	if (controller.controller) {
		action = controller.action
		controller = controller.controller
	} else if (controller.indexOf("#") >= 0) {
		controller = controller.split("#")
		action = controller[1]
		controller = controller[0]
	}

	this._controller = controller
	this._action = action

	return this
}

Route.prototype.compileUrl = function () {
	if (!this.url) {
		return
	}

	// convert url to parseable format
	this._regexpUrl = xre("^" + this._url.replace(UrlRegExp, "(?<$1>[^/]+)") + "$")
}

Route.prototype.generate = function (params) {
	return strformat(this._url, params)
}

Route.prototype.parse = function (method, url) {
	if ((this._method !== "any" && method !== this._method) || !this._regexpUrl.test(url)) {
		return false
	}

	var results = this._regexpUrl.exec(url)
	results.controller = this._controller
	results.action = this._action
	return results
}

module.exports = Route
