var fs = require("fs")
var path = require("path")
var co = require("co")

exports.requireDir = function (dir, dontThrow) {
	var res = {}

	if (fs.existsSync(dir)) {
		fs.readdirSync(dir).forEach(function (file) {
			try {
				var mod = require(path.join(dir, file))
				res[path.basename(file, path.extname(file))] = mod
			} catch (ex) {
				if (!dontThrow) {
					throw ex
				}
			}
		})
	} else if (!dontThrow) {
		throw new Error("dir " + dir + " not found!")
	}

	return res
}

exports.create = function (ctor, args) {
	var obj = Object.create(ctor)
	var inst = ctor.apply(obj, args)

	// are we a custom object or a native one?
	return Object(inst) === inst? inst: obj
}

exports.isGeneratorFunction = function (f) {
	return obj && obj.constructor && 'GeneratorFunction' === obj.constructor.name
}

exports.coApply = function (f, args, ctx) {
	if (arguments.length === 2 && !args.hasOwnProperty("length")) {
		ctx = args
		args = []
	}
	if (!exports.isGeneratorFunction(f)) {
		return f.apply(ctx, args)
	} else {
		return co(f).apply(ctx, args)
	}
}
